/**************************************************
 * VRML 2.0, Draft 2 Parser
 * Copyright (C) 1996 Silicon Graphics, Inc.
 *
 * Author(s)    : Gavin Bell
 *                Daniel Woods (first port)
 **************************************************
 */

//
// The VrmlNodeType class is responsible for storing information about node
// or prototype types.
//

#include <list.h>

//using namespace std;

class VrmlNodeType {
  public:
  	// Constructor.  Takes name of new type (e.g. "Transform" or "Box")
    // Copies the string given as name.
    VrmlNodeType(const char *nm);

    // Destructor exists mainly to deallocate storage for name
    ~VrmlNodeType();

    // Namespace management functions.  PROTO definitions add node types
    // to the namespace.  PROTO implementations are a separate node
    // namespace, and require that any nested PROTOs NOT be available
    // outside the PROTO implementation.
    // addToNameSpace will print an error to stderr if the given type
    // is already defined.
    static void addToNameSpace(VrmlNodeType *);
    static void pushNameSpace();
    static void popNameSpace();

    // Find a node type, given its name.  Returns NULL if type is not defined.
    static const VrmlNodeType *find(const char *nm);

    // Routines for adding/getting eventIns/Outs/fields
    void addEventIn(const char *name, int type);
    void addEventOut(const char *name, int type);
    void addField(const char *name, int type);
    void addExposedField(const char *name, int type);

    int hasEventIn(const char *name) const;
    int hasEventOut(const char *name) const;
    int hasField(const char *name) const;
    int hasExposedField(const char *name) const;
    
    const char *getName() const { return name; }

    typedef struct {
    	char *name;
        int type;
    } NameTypeRec;
        
  private:
    void add(/*std::*/list<NameTypeRec*> &,const char *,int);
    int has(const /*std::*/list<NameTypeRec*> &,const char *) const;

    char *name;

    // Node types are stored in this data structure:
    static list<VrmlNodeType*> typeList;

    /*std::*/list<NameTypeRec*> eventIns;
    /*std::*/list<NameTypeRec*> eventOuts;
    /*std::*/list<NameTypeRec*> fields;
};
